{% extends "base.html" %} {% block content %} {% if metadata.show_title %} {%
include "title.html" %} {% if metadata.date.date() %}
<center>
  <time class="post-date" datetime="{{ metadata.date.date() | safe }}"
  >{{ metadata.date.date() | safe}}</time>
</center>
{% endif %} {% endif %} {% if is_post %} {% include "post_nav.html" %} {% endif
%}
<hr>

<article>{{ content | safe }}</article>
{% endblock %}
