{% extends "base.html" %} {% block content %} {% if metadata.show_title %} {%
include "title.html" %} {% endif %}

<article>{{ content | safe }}</article>

{% if post_list %}
<div class="post-list">
  <ul>
    {% for item in post_list %}
    <li>
      <time class="post-list-date" datetime="{{ item.metadata.date.date() | safe }}"
      >{{ item.metadata.date.date() | safe}}</time>: <a href="/{{ item.url }}"
      >{{ item.metadata.title }}</a>
    </li>
    {% endfor %}
  </ul>
</div>
{% endif %} {% endblock %}
